<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services\CadVendedor as CadVendedorService;
use \Exception as Exception;

class CadVendedor extends Crud {
    
    protected $dataSourceName = 'vnd06400_cadvendedor'; 
    protected $dataSourceAddress = 'vnd06400_endevende'; 
    protected $dataSourceManager;
    protected $cadvendedorservice;
    
    public function __construct(Manager $dataSourceManager, CadVendedorService $cadvendedorservice ) {
        $this->dataSourceManager = $dataSourceManager;
        $this->cadvendedorservice = $cadvendedorservice;
    }
    
    // Method called by the frontend 
    public function saveCadVendedor(DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        $row = $rows[0];
        $row['IDTPIJURVEN'] = $this->AjustaInscricao($row['NRINSJURVEN']);
        $row['IDVRINCIDECONTA'] = ($row['IDVRINCIDECONTA'] === 'N') ? 'N' : 'S'; 

        $validOperador = $this -> cadvendedorservice -> validOperador($row['CDVENDEDOR']);
        
        if (empty($validOperador)){
            // Save data into table VENDEDOR
            try {
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceName);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
                $nrPersistedRows = count($persistedRows);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
            
            $__is_new = $this->InsertOrUpdate($row);
            
            // If he already has or want to change his address
            if( $row['hasAddress'] || !($__is_new)) {
                $this->saveEndereco( $dataSet, $response, $__is_new);
            }
        } else{
           $response->setError(new DTO\Response\Error('Operador já está relacionado a outro vendedor!',1));
        }
    }
    
    // Save data into table ENDEVENDE
    private function saveEndereco( $dataSet, $response, $__is_new) {
        
        $row = $dataSet->getRows()[0];
        // Then dataSourceManager will know if it's an Update or Insert operation.
        $row['__is_new'] = $__is_new;
        
        // Seta os campos de auditoria
        if ( $row['__is_new'] ) {
            $row['DTINCLUSAO']     = $row['DTULTATU']     == null ? $row['DTINCLUSAO']     : $row['DTULTATU'];
            $row['CDOPERINCLUSAO'] = $row['CDOPERULTATU'] == null ? $row['CDOPERINCLUSAO'] : $row['CDOPERULTATU'];
            $row['NRORGINCLUSAO']  = $row['NRORGULTATU']  == null ? $row['NRORGINCLUSAO']  : $row['NRORGULTATU'];
        }
        
        /*
        // Explanation of the next if
        
            -> Is it a new address? 
            	-Yes
            	-> Can Save ?
            		- Yes: Save 
            		- No: Do Nothing
            	-No
            	-> Can Save? 
            		- Yes: Save
            		- No: Delete the old one
        
        */
        
        // If the vendor has an address, saved, but now he can't save, it's because he wants to delete it.
        if(!$row['canSaveAddress'] && !$row['__is_new']) {
            try {
                $dataSet->setDataSourceName($this->dataSourceAddress);
                $deletedRows = $this->dataSourceManager->delete($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceAddress, $deletedRows));
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        } else if($row['canSaveAddress']) {
            try {
                $dataSet->setDataSourceName($this->dataSourceAddress);
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    // Set IDTPIJURVEN property
    private function AjustaInscricao($stInscricao) {
        if (empty($stInscricao) || (strlen($stInscricao) > 20)) {
            throw new Exception('invalidParameters');
        }
        $len = strlen($stInscricao);
        
        if($len === 11) {
            return 'F';     // Pessoa Física
        } else if($len === 14) {
            return 'J';     // Pessoa Jurídica
        } else if($len <= 5) {
            return 'O';     // Código Genérico    
        } else {
            return 'I';     // Inscrição Internacional    
        }       
    }

    // Fix the insert in two tables problem 
    private function InsertOrUpdate($data) {
        // Searches for an Vendor Address registry
     	$resultSet = $this->cadvendedorservice->getVendorAddress($data['CDVENDEDOR'], $data['NRORG']);
        return ( ($resultSet === null) || (count($resultSet) === 0) );
    }

    public function getVendor(DTO\Request\Row $request, DTO\Response $response) {
        $data = $request->getRow();
        $response->addDataSet( new DataSet( $this->dataSourceName, $this->cadvendedorservice->getVendor($data['CDVENDEDOR']) ));    
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        
        try {
            $dataSet = $request->getDataSet();
            
            // Delete from ENDEVENDE Table
            $dataSet->setDataSourceName($this->dataSourceAddress);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceAddress, $deletedRows));
            
            // Delete from VENDEDOR Table
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $nrDeletedRows = count($deletedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error(
                ($e->getCode() === 0) 
                ? 'Não foi possível deletar, pois existem registros filhos.'
                : $e->getMessage(), $e->getCode()));
        }
    }

}